
// SYSLOOP® Serial Protocol © Copyright 2016 EMPYREUM® Information Technologies. All rights reserved.
// This work (ideas, program logic, algorithms, systems, methods, concepts, or layouts) is the intellectual property of EMPYREUM®.

#include "COBS.h"
#include "arr.h"
#include "log.h"
#include <stdlib.h>

bool COBSEncode(void** output, uint64_t* outputSize, const void* input, uint64_t inputSize) {
	uint8_t* o = NULL; uint64_t os = 0, oc = 0;
	const uint8_t *i = input, *il = i + inputSize;
	uint8_t c = 0x01; uint64_t ci = os++;
	while(i != il) {
		if(!arrexp((void**)&o, &os, &oc, 3)) goto error;
		if(*i == 0x00) {
			*(o + ci) = c, c = 0x01, ci = os++;
		} else {
			*(o + os++) = *i;
			if(++c == 0xff)
				*(o + ci) = c, c = 0x01, ci = os++;
		}
		i++;
	}
	if(!arrexp((void**)&o, &os, &oc, 2)) goto error;
	*(o + ci) = c;
	*(o + os++) = 0x00;
	if(arrcmp((void**)&o, &os, &oc)) goto error;
	*output = o, *outputSize = os;
	return false;

error:;
	free(o);
	return true;
}

bool COBSDecode(void** output, uint64_t* outputSize, const void* input, uint64_t inputSize) {
	uint8_t* o = NULL; uint64_t os = 0, oc = 0;
	const uint8_t *i = input, *il = i + inputSize;
	if(i == il || *--il != 0x00) { log("corrupt"); goto error; }
	while(i != il) {
		uint8_t c = *i++;
		if((i + c - 1) > il) { log("corrupt"); log("c=%u %016llx + c - 1 > %016llx", c, (unsigned long long)(uintptr_t)i, (unsigned long long)(uintptr_t)il); goto error; }
		if(!arrexp((void**)&o, &os, &oc, c)) goto error;
		uint8_t j; for(j = 1; j < c; j++) *(o + os++) = *i++;
		if(c < 0xff && i != il) *(o + os++) = 0x00;
	}
	if(arrcmp((void**)&o, &os, &oc)) goto error;
	*output = o, *outputSize = os;
	return false;

error:;
	free(o);
	return true;
}
