
// SYSLOOP® Serial Protocol © Copyright 2016 EMPYREUM® Information Technologies. All rights reserved.
// This work (ideas, program logic, algorithms, systems, methods, concepts, or layouts) is the intellectual property of EMPYREUM®.

#include "log.h"
#include <stdlib.h>

void dump(const char* text, const void* array, uint64_t arraySize) {
	printf("%-16s", text);
	const uint8_t *a = array, *al = a + arraySize;
	while(a != al) printf(" %02X", *a++);
	printf("\n");
}

int main(int ac, char** av) {

	// data
	uint8_t data[] = {  0xad, 0xde, 0x00, 0x00, 0x00, 0xce, 0xfa, 0x00, }; uint64_t dataSize = sizeof(data);
	uint8_t dataChecksum = CRC8(data, dataSize);

	// payload with checksum
	uint64_t payloadSize = dataSize + 1; uint8_t payload[payloadSize];
	memcpy(payload, data, dataSize); payload[dataSize] = dataChecksum;

	// encode
	uint8_t* packet; uint64_t packetSize;
	if(!COBSEncode(&packet, &packetSize, payload, payloadSize)) {


		dump("data", data, dataSize);
		dump("packet", packet, packetSize);

		log("... transmit - receive ...");

		// decode
		uint8_t* _payload; uint64_t _payloadSize;
		if(!COBSDecode(&_payload, &_payloadSize, packet, packetSize)) {
			if(_payloadSize > 0) {
				const void* _data = _payload; uint64_t _dataSize = _payloadSize - 1;
				uint8_t _dataChecksum = CRC8(_data, _dataSize);
				if(_payload[_dataSize] == _dataChecksum) {
					dump("data", _data, _dataSize);
					if(_dataSize != dataSize || memcmp(data, _data, _dataSize)) log("internal error");
				} else log("packet corrupt (checksum)");
			}
			free(_payload);
		}
		free(packet);
	}

	return 0;
}
